''' <summary>
''' Classe de gestion de la connexion UDP
''' </summary>
''' <remarks></remarks>
Friend NotInheritable Class ClsUdp
    Inherits ClsBaseThread

#Region "Classes"
    ''' <summary>
    ''' Classe identifiant un client distant
    ''' </summary>
    ''' <remarks></remarks>
    Friend Class ClsDistant

        ''' <summary>
        ''' Point de connexion distant
        ''' </summary>
        ''' <remarks></remarks>
        Private mIP As Net.IPEndPoint
        ''' <summary>
        ''' Pseudo du client
        ''' </summary>
        ''' <remarks></remarks>
        Private mPseudo As String
        ''' <summary>
        ''' Dernire activit
        ''' </summary>
        ''' <remarks></remarks>
        Private mLastRefresh As Integer

        ''' <summary>
        ''' Constructeur de la classe
        ''' </summary>
        ''' <param name="pIp">Ip de client distant</param>
        ''' <param name="pPseudo">Pseudo du client</param>
        ''' <remarks></remarks>
        Public Sub New(ByVal pIp As Net.IPEndPoint, ByVal pPseudo As String)
            mIP = pIp
            mPseudo = pPseudo
            RAZRefresh()
        End Sub

        ''' <summary>
        ''' Signalement de l'activit du client
        ''' </summary>
        ''' <remarks></remarks>
        Public Sub RAZRefresh()
            mLastRefresh = Environment.TickCount
        End Sub
        ''' <summary>
        ''' Retourne une chaine reprsentant le client
        ''' </summary>
        ''' <returns></returns>
        ''' <remarks></remarks>
        Public Overrides Function ToString() As String
            If My.Settings.ShowClientIP Then
                Return Pseudo + " (" + IP.Address.ToString + ")"
            Else
                Return Pseudo
            End If
        End Function

        ''' <summary>
        ''' Retourne un boulen indiquant si le client a expir
        ''' </summary>
        ''' <returns></returns>
        ''' <remarks></remarks>
        Public Function Expired() As Boolean
            Dim lBoo As Boolean = (Environment.TickCount - LastRefresh) < My.Settings.NbRefreshBeforeDel * My.Settings.IntervalRefreshList * 1000
            If lBoo = False AndAlso msLstDistant.Contains(Me) Then
                AddLog("ClsDistant", "Suppression du client : {0}", Me, EPrio.Information)
                msLstDistant.Remove(Me)
                UdpRaiseEvent()
            End If
            Return lBoo
        End Function
        ''' <summary>
        ''' Retourne un boulen indiquant si il s'agit de l'objet recherch
        ''' </summary>
        ''' <param name="pPseudo">Pseudo recherch</param>
        ''' <param name="pIp">Ip distante</param>
        ''' <returns></returns>
        ''' <remarks></remarks>
        Public Overloads Function Equals(ByVal pPseudo As String, ByVal pIp As Net.IPEndPoint) As Boolean
            Return mPseudo = pPseudo And mIP.ToString = pIp.ToString
        End Function

        ''' <summary>
        ''' Point de connexion distant
        ''' </summary>
        ''' <remarks></remarks>
        Public ReadOnly Property IP() As Net.IPEndPoint
            Get
                Return mIP
            End Get
        End Property
        ''' <summary>
        ''' Pseudo du client
        ''' </summary>
        ''' <remarks></remarks>
        Public Property Pseudo() As String
            Get
                Return mPseudo
            End Get
            Set(ByVal value As String)
                mPseudo = value
            End Set
        End Property
        ''' <summary>
        ''' Dernire activit
        ''' </summary>
        ''' <remarks></remarks>
        Public ReadOnly Property LastRefresh() As Integer
            Get
                Return mLastRefresh
            End Get
        End Property
    End Class
    ''' <summary>
    ''' Classe de mmorisation d'un message
    ''' </summary>
    ''' <remarks></remarks>
    Friend Class ClsMessage

        ''' <summary>
        ''' Client distant
        ''' </summary>
        ''' <remarks></remarks>
        Private mDistant As ClsDistant
        ''' <summary>
        ''' Message
        ''' </summary>
        ''' <remarks></remarks>
        Private mMessage As String
        ''' <summary>
        ''' Heure de rception
        ''' </summary>
        ''' <remarks></remarks>
        Private mHeure As Date

        ''' <summary>
        ''' Constructeur de la classe
        ''' </summary>
        ''' <param name="pMessage">Message</param>
        ''' <param name="pSender">Client emmeteur</param>
        ''' <remarks></remarks>
        Public Sub New(ByVal pMessage As String, ByVal pSender As ClsDistant)
            mDistant = pSender
            mMessage = pMessage
            mHeure = Now
        End Sub

        ''' <summary>
        ''' Retourne le client distant
        ''' </summary>
        ''' <value></value>
        ''' <returns></returns>
        ''' <remarks></remarks>
        Public ReadOnly Property Distant() As ClsDistant
            Get
                Return mDistant
            End Get
        End Property
        ''' <summary>
        ''' Retourne le message
        ''' </summary>
        ''' <value></value>
        ''' <returns></returns>
        ''' <remarks></remarks>
        Public ReadOnly Property Message() As String
            Get
                Return mMessage
            End Get
        End Property
        ''' <summary>
        ''' Retourne l'heure de rception du message
        ''' </summary>
        ''' <value></value>
        ''' <returns></returns>
        ''' <remarks></remarks>
        Public ReadOnly Property Heure() As Date
            Get
                Return mHeure
            End Get
        End Property
    End Class
#End Region

#Region "Shared"

#Region "Attributs"
    ''' <summary>
    ''' Mmorise l'objet de thread
    ''' </summary>
    ''' <remarks></remarks>
    Private Shared msObject As ClsUdp = Nothing
    ''' <summary>
    ''' Mmorise la liste des messages reus
    ''' </summary>
    ''' <remarks></remarks>
    Private Shared msLstMessage As New List(Of ClsMessage)
    ''' <summary>
    ''' Mmorise la liste des clients
    ''' </summary>
    ''' <remarks></remarks>
    Private Shared msLstDistant As New List(Of ClsDistant)
    ''' <summary>
    ''' Mmorise le pseudo du client local
    ''' </summary>
    ''' <remarks></remarks>
    Private Shared msPseudo As String = ""
#End Region

    ''' <summary>
    ''' Constructeur static
    ''' </summary>
    ''' <remarks></remarks>
    Shared Sub New()
        AddHandler MdlFriend.NetworkAvailabilityChanged, New MdlFriend.EvtNetworkAvailabilityChanged(AddressOf GestEvtNetwork)

        If MdlFriend.NetworkAvailable Then
            msObject = New ClsUdp
        End If

        msPseudo = My.Settings.Pseudo
        If msPseudo = "" Then
            My.Settings.Pseudo = My.User.Name.Replace("\"c, "."c)
            msPseudo = My.Settings.Pseudo
        End If
    End Sub

#Region "Membres publiques"
    ''' <summary>
    ''' Retourne l'objet de la classe (singleton)
    ''' </summary>
    ''' <value></value>
    ''' <returns></returns>
    ''' <remarks></remarks>
    Private Shared ReadOnly Property UDP() As ClsUdp
        Get
            If msObject IsNot Nothing Then
                If msObject.IsRun Then
                    Return msObject
                Else
                    msObject = Nothing
                    If MdlFriend.NetworkAvailable Then
                        msObject = New ClsUdp
                        Return msObject
                    Else
                        Return Nothing
                    End If
                End If
            ElseIf MdlFriend.NetworkAvailable Then
                msObject = New ClsUdp
                Return msObject
            Else
                Return Nothing
            End If
        End Get
    End Property
    ''' <summary>
    ''' Retourne la liste des clients distants
    ''' </summary>
    ''' <value></value>
    ''' <returns></returns>
    ''' <remarks></remarks>
    Public Shared ReadOnly Property Distants() As ObjectModel.ReadOnlyCollection(Of ClsDistant)
        Get
            Return msLstDistant.AsReadOnly
        End Get
    End Property
    ''' <summary>
    ''' Retourne un boulen indiquant si le programme est connect
    ''' </summary>
    ''' <value></value>
    ''' <returns></returns>
    ''' <remarks></remarks>
    Public Shared ReadOnly Property IsConnected() As Boolean
        Get
            Return MdlFriend.NetworkAvailable AndAlso UDP IsNot Nothing AndAlso UDP.mUdp IsNot Nothing
        End Get
    End Property
    ''' <summary>
    ''' Retourne la liste des messages
    ''' </summary>
    ''' <returns></returns>
    ''' <remarks></remarks>
    Public Shared ReadOnly Property Messages() As List(Of ClsMessage)
        Get
            Return msLstMessage
        End Get
    End Property

    ''' <summary>
    ''' Retourne ou dfinit le pseudo du client local
    ''' </summary>
    ''' <value></value>
    ''' <returns></returns>
    ''' <remarks></remarks>
    Public Shared Property Pseudo() As String
        Get
            Return msPseudo
        End Get
        Set(ByVal value As String)
            If value = msPseudo Or value.Length = 0 Then Exit Property
            Dim lAnc As String = msPseudo
            My.Settings.Pseudo = value
            msPseudo = value
            AddLog("UDP", "Changement de pseudo : {0} -> {1}", lAnc, value, EPrio.Information)
            Try
                RaiseEvent ChangePseudo(msPseudo, lAnc)
            Catch ex As Exception
                AddLog("ClsUdp.set_Pseudo", "Exception lors de l'event : {0}", ex, EPrio.Avertissement)
#If DEBUG Then
                Stop
#End If
            End Try
        End Set
    End Property

    ''' <summary>
    ''' Envoie un message sur le rseau
    ''' </summary>
    ''' <param name="pMsg">Message</param>
    ''' <param name="pDest">Destinataire(s) spars par des ";"</param>
    ''' <param name="pAck">Optionnel, demande d'acquittement</param>
    ''' <remarks></remarks>
    Public Shared Sub SendMessage(ByVal pMsg As String, ByVal pDest As String, Optional ByVal pAck As Boolean = False)
        Dim lUdp As ClsUdp = UDP
        If lUdp IsNot Nothing Then lUdp.SendMessage(msPseudo, pMsg, pDest, pAck)
    End Sub
    ''' <summary>
    ''' Rafraichis les donnes
    ''' </summary>
    ''' <remarks></remarks>
    Public Shared Sub RefreshState()
        Try
            Dim lUdp As ClsUdp = UDP
            Dim lCnt As Integer = msLstDistant.Count
            For i As Integer = msLstDistant.Count - 1 To 0 Step -1
                msLstDistant(i).Expired()
            Next
        Catch ex As Exception
            AddLog("ClsUdp.RefreshState", "Exception non gre : {0}", ex, EPrio.Avertissement)
        End Try
    End Sub
    ''' <summary>
    ''' Redmarre l'interface rseau
    ''' </summary>
    ''' <remarks></remarks>
    Public Shared Sub RestartNetwork()
        AddLog("ClsUdp", "Redmarrage de l'interface rseau", EPrio.Information)
        If UDP IsNot Nothing Then UDP.StopThread(True)
        Dim lUdp As ClsUdp = UDP
    End Sub
#End Region

#Region "Membres privs"
    ''' <summary>
    ''' Gre l'event de changement d'accessibilit du rseau
    ''' </summary>
    ''' <remarks></remarks>
    Private Shared Sub GestEvtNetwork()
        If MdlFriend.NetworkAvailable And msObject Is Nothing Then
            msObject = New ClsUdp
        ElseIf MdlFriend.NetworkAvailable = False And msObject IsNot Nothing Then
            msObject.StopThread()
        End If
    End Sub
    ''' <summary>
    ''' Gre l'arrive d'un message
    ''' </summary>
    ''' <param name="pSender"></param>
    ''' <param name="pMessage"></param>
    ''' <remarks></remarks>
    Private Shared Sub ClsUdp_MessageReceive(ByVal pSender As ClsUdp, ByVal pMessage As ClsMessage) Handles Me.MessageReceive
        msLstMessage.Add(pMessage)
    End Sub
    ''' <summary>
    ''' Ajoute le client distant  la liste
    ''' </summary>
    ''' <param name="pDistant">Nouveau client</param>
    ''' <returns></returns>
    ''' <remarks></remarks>
    Private Shared Function Add(ByVal pDistant As ClsDistant) As ClsDistant
        msLstDistant.Add(pDistant)
        Return pDistant
    End Function
#End Region

#Region "Events publiques"

#Region "Delegates"
    ''' <summary>
    ''' Delegate pour l'event MessageReceive
    ''' </summary>
    ''' <param name="pSender">Emmeteur de l'event</param>
    ''' <param name="pMessage">Message reu</param>
    ''' <remarks></remarks>
    Public Delegate Sub EvtMessageReceive(ByVal pSender As ClsUdp, ByVal pMessage As ClsMessage)
    ''' <summary>
    ''' Delegate pour l'event ChangeDistant
    ''' </summary>
    ''' <param name="pDistant">Tableau des clients distant</param>
    ''' <remarks></remarks>
    Public Delegate Sub EvtChangeDistant(ByVal pDistant As Collections.ObjectModel.ReadOnlyCollection(Of ClsDistant))
    ''' <summary>
    ''' Delegate pour l'event Ack
    ''' </summary>
    ''' <param name="pSender">Emmeteur de l'event</param>
    ''' <param name="pDistant">Client distant ayant gnr l'ack</param>
    ''' <remarks></remarks>
    Public Delegate Sub EvtAck(ByVal pSender As ClsUdp, ByVal pDistant As ClsDistant)
    ''' <summary>
    ''' Delegate pour l'event ChangePseudo
    ''' </summary>
    ''' <param name="pNewPseudo">Nouveau pseudo</param>
    ''' <param name="pAncPseudo">Ancien pseudo</param>
    ''' <remarks></remarks>
    Public Delegate Sub EvtChangePseudo(ByVal pNewPseudo As String, ByVal pAncPseudo As String)
#End Region

#Region "Events"
    ''' <summary>
    ''' Evenement gnr lors de l'arrive d'un message
    ''' </summary>
    ''' <remarks></remarks>
    Public Shared Event MessageReceive As EvtMessageReceive
    ''' <summary>
    ''' Evenement gnr lors de la connexion/deconnexion/renomage d'un client distant
    ''' </summary>
    ''' <remarks></remarks>
    Public Shared Event ChangeDistant As EvtChangeDistant
    ''' <summary>
    ''' Evenement gnr lors de l'arrive d'un acquittement
    ''' </summary>
    ''' <remarks></remarks>
    Public Shared Event Ack As EvtAck
    ''' <summary>
    ''' Evenement gnr lors du changement du pseudo du client local
    ''' </summary>
    ''' <remarks></remarks>
    Public Shared Event ChangePseudo As EvtChangePseudo
#End Region
#End Region
#End Region

#Region "Non Shared"

    ''' <summary>
    ''' Socket UDP
    ''' </summary>
    ''' <remarks></remarks>
    Private mUdp As Net.Sockets.UdpClient = Nothing

    ''' <summary>
    ''' Cosntructeur de la classe
    ''' </summary>
    ''' <remarks></remarks>
    Public Sub New()
        MyBase.New("Rception UDP")
    End Sub

    ''' <summary>
    ''' Envoie un message sur le rseau
    ''' </summary>
    ''' <param name="pPseudo">Pseudo du client</param>
    ''' <param name="pMsg">Message</param>
    ''' <param name="pDest">Destinateires</param>
    ''' <param name="pAck">Demande d'acquittement</param>
    ''' <remarks></remarks>
    Public Sub SendMessage(ByVal pPseudo As String, ByVal pMsg As String, ByVal pDest As String, ByVal pAck As Boolean)
        Dim ltData As New List(Of Byte)
        'Envoie du pseudo
        ltData.Add(3)
        ltData.AddRange(GetByteArray(pPseudo))
        ltData.AddRange(GetByteArray(pDest))
        ltData.AddRange(GetByteArray(pMsg))
        If pAck Then ltData.Add(0)
        Send(ltData)
    End Sub

    ''' <summary>
    ''' Gnre l'event ChangeDistant
    ''' </summary>
    ''' <remarks></remarks>
    Private Shared Sub UdpRaiseEvent()
        Try
            RaiseEvent ChangeDistant(Distants)
        Catch ex As Exception
            AddLog("ClsUpd.Event", "Exception dans ChangeDistant : {0}", ex.ToString, EPrio.Avertissement)
        End Try
    End Sub
    ''' <summary>
    ''' Gnre l'event MessageReceive
    ''' </summary>
    ''' <param name="pMsg">Message reu</param>
    ''' <remarks></remarks>
    Private Sub UdpRaiseEvent(ByVal pMsg As ClsMessage)
        Try
            RaiseEvent MessageReceive(Me, pMsg)
        Catch ex As Exception
            AddLog("ClsUdp.Event", "Exception dans MessageReceive : {0}", ex.ToString, EPrio.Avertissement)
        End Try
    End Sub
    ''' <summary>
    ''' Gnre l'event Ack
    ''' </summary>
    ''' <param name="pDistant">Client distant ayant fait l'acquittement</param>
    ''' <remarks></remarks>
    Private Sub UdpRaiseEvent(ByVal pDistant As ClsDistant)
        Try
            RaiseEvent Ack(Me, pDistant)
        Catch ex As Exception
            AddLog("ClsUdp.Event", "Exception dans Ack : {0}", ex.ToString, EPrio.Avertissement)
        End Try
    End Sub


    Private Sub ClsUdp_ChangePseudo(ByVal pNewPseudo As String, ByVal pAncPseudo As String) Handles Me.ChangePseudo
        Dim ltData As New List(Of Byte)
        'Envoie du nouveau pseudo
        ltData.Add(2)
        ltData.AddRange(GetByteArray(pAncPseudo))
        ltData.AddRange(GetByteArray(pNewPseudo))
        Send(ltData)
    End Sub

    ''' <summary>
    ''' Retourne le client distant
    ''' </summary>
    ''' <param name="pPseudo">Pseudo  rechercher</param>
    ''' <param name="pIp">Ip distante</param>
    ''' <returns></returns>
    ''' <remarks></remarks>
    Private Function GetDistant(ByVal pPseudo As String, ByVal pIp As Net.IPEndPoint) As ClsDistant
        For Each lDist As ClsDistant In Distants
            If lDist.Equals(pPseudo, pIp) Then
                Return lDist
            End If
        Next
        Return Nothing
    End Function


    Private Sub Send(ByVal pData As List(Of Byte))
        Send(pData, New Net.IPEndPoint(Net.IPAddress.Parse(My.Settings.IPListen), My.Settings.PortListen))
    End Sub

    Private Sub Send(ByVal pData As List(Of Byte), ByVal pDest As Net.IPEndPoint)
        If mUdp IsNot Nothing Then
            Try
                mUdp.Send(pData.ToArray, pData.Count, pDest)
            Catch ex As Exception
                AddLog("SendData", "Exception lors de l'envoie : {0}", ex, EPrio.Avertissement)
            End Try
        End If
    End Sub


    Protected Overrides Sub ProSubThread()
        If Not mCanRun Then Exit Sub
        Dim ltData As New List(Of Byte)

        mUdp = New Net.Sockets.UdpClient()
        'Autorise d'autre programmes  utiliser ce port (pour d'autres fonctions)
        mUdp.ExclusiveAddressUse = False
        'Lie le socket au port
        mUdp.Client.Bind(New Net.IPEndPoint(Net.IPAddress.Parse("0.0.0.0"), My.Settings.PortListen))
        '224.0.0.0 through 239.255.255.255 
        If CInt(My.Settings.IPListen.Split("."c)(0)) >= 224 And CInt(My.Settings.IPListen.Split("."c)(0)) <= 239 Then
            mUdp.JoinMulticastGroup(Net.IPAddress.Parse(My.Settings.IPListen), My.Settings.IPTTL)
            mUdp.EnableBroadcast = False
        End If

        'Envoie du pseudo
        ltData.Add(2)
        ltData.Add(0)
        ltData.AddRange(GetByteArray(Pseudo))
        Send(ltData)

        'Envoie de la requte de rception
        ltData.Clear()
        ltData.Add(0)
        ltData.AddRange(GetByteArray("*"))
        Send(ltData)

        Dim lLastRefresh As Integer = 0

        Do While True
            If mUdp.Available > 0 Then
                ltData.Clear()
                Dim lRemote As New Net.IPEndPoint(Net.IPAddress.Any, 7000)
                ltData.AddRange(mUdp.Receive(lRemote))

                Select Case ltData(0)
                    Case 0 'Recherche hosts
                        ltData.RemoveAt(0)
                        Dim lRecherche As String = GetString(ltData)
                        If Pseudo Like lRecherche Then
                            ltData.Clear()
                            ltData.Add(1)
                            ltData.AddRange(GetByteArray(Pseudo))
                            ltData.Add(0)
                            Send(ltData, lRemote)
                        End If
                    Case 1 'Reply 00h
                        ltData.RemoveAt(0)
                        Dim lRecherche As String = GetString(ltData)
                        If lRecherche <> "" Then
                            Dim lDist As ClsDistant = GetDistant(lRecherche, lRemote)
                            If lDist IsNot Nothing Then
                                lDist.RAZRefresh()
                            Else
                                AddLog("ReceptUdp", "Ajout d'un client : {0}", Add(New ClsDistant(lRemote, lRecherche)), EPrio.Basse)
                                UdpRaiseEvent()
                            End If
                        End If
                    Case 2 'ChangePseudo
                        Dim lAnc As String = ""
                        Dim lNew As String = ""
                        ltData.RemoveAt(0)
                        lAnc = GetString(ltData)
                        lNew = GetString(ltData)

                        If lNew = "" And lAnc <> "" Then
                            Dim lDist As ClsDistant = GetDistant(lAnc, lRemote)
                            If lDist IsNot Nothing Then msLstDistant.Remove(lDist)


                        ElseIf lNew <> "" And lAnc = "" Then
                            AddLog("ReceptUdp", "Ajout d'un client : {0}", Add(New ClsDistant(lRemote, lNew)), EPrio.Basse)
                        Else
                            Dim lDist As ClsDistant = GetDistant(lAnc, lRemote)
                            If lDist IsNot Nothing Then
                                lDist.RAZRefresh()
                                lDist.Pseudo = lNew
                                AddLog("ReceptUdp", "Changement d'un client : " + lAnc + " -> " + lNew, EPrio.Basse)
                            End If
                        End If
                        UdpRaiseEvent()
                    Case 3 'Message
                        Dim lSrc As String = ""
                        Dim lDest As String = ""
                        Dim lMsg As String = ""
                        Dim lAck As Boolean = False

                        ltData.RemoveAt(0)
                        lSrc = GetString(ltData)
                        lDest = GetString(ltData)
                        lMsg = GetString(ltData)
                        If ltData.Count > 0 Then lAck = True

                        Dim lDist As ClsDistant = GetDistant(lSrc, lRemote)

                        If lDest = "" Then
                            lDest = Pseudo
                        Else
                            Dim ltDest As String() = lDest.Split(";"c)

                            For Each llDest As String In ltDest
                                If msPseudo.ToLower.IndexOf(llDest.ToLower) <> -1 Or llDest.ToLower = msPseudo.ToLower Then
                                    lDest = Pseudo
                                End If
                            Next
                        End If

                        If lDest <> Pseudo Then Continue Do

                        UdpRaiseEvent(New ClsMessage(lMsg, lDist))

                        If lAck Then
                            ltData.Clear()
                            ltData.Add(4)
                            ltData.AddRange(GetByteArray(Pseudo))
                            ltData.AddRange(GetByteArray(lDist.Pseudo))
                            Send(ltData, lDist.IP)
                        End If
                    Case 4 'Ack
                        Dim lSrc As String = ""
                        Dim lDest As String = ""

                        ltData.RemoveAt(0)

                        lSrc = GetString(ltData)
                        lDest = GetString(ltData)

                        Dim lDist As ClsDistant = GetDistant(lSrc, lRemote)
                        If lDist Is Nothing Then
                            lDist = New ClsDistant(lRemote, lSrc)
                            AddLog("ReceptUdp", "Ajout d'un client : {0}", Add(lDist), EPrio.Basse)
                            UdpRaiseEvent()
                        End If

                        AddLog("ReceptUdp", "Ack d'un client : " + lDist.ToString, EPrio.Basse)
                        UdpRaiseEvent(lDist)
                End Select
            Else 'Aucune rception, on s'occupe des taches moins importantes
                Threading.Thread.Sleep(100)
                Dim lAttnt As Integer = My.Settings.IntervalRefreshList * 1000
                If (Environment.TickCount - lLastRefresh) > lAttnt Then
                    'Envoie de la requte de rception
                    ltData.Clear()
                    ltData.Add(0)
                    ltData.AddRange(GetByteArray("*"))
                    ltData.Add(0)
                    Send(ltData)
                    'Suppression des nom expirs
                    RefreshState()
                    lLastRefresh = Environment.TickCount
                End If
            End If
            If Not mCanRun Then Exit Do
        Loop

        CloseConnect()
    End Sub

    Protected Overrides Function ProErrThread(ByVal pEx As System.Exception) As Boolean
        Dim lCloseApp As Boolean = False
        Dim lRestart As Boolean = False
        If TypeOf pEx Is Threading.ThreadAbortException Then
            AddLog("Thread UDP", "thread avort", EPrio.Avertissement)
        ElseIf TypeOf pEx Is Net.Sockets.SocketException Then
            Select Case DirectCast(pEx, Net.Sockets.SocketException).NativeErrorCode
                Case 10065 'non connect  un rseau (physique)
                    lRestart = True
                    AddLog("Thread UDP", "Non connect  un rseau physique", EPrio.Avertissement)
                Case 10048 'Interface utilise
                    AddLog("Thread UDP", "Interface rseau dj utilise", EPrio.Avertissement)
                    If MessageBox.Show("Le port est dj utilis par une autre application, voulez-vous arrter l'application ?", "Erreur UDP", MessageBoxButtons.YesNo, MessageBoxIcon.Error) = DialogResult.Yes Then
                        lCloseApp = True
                    Else
                        lRestart = True
                    End If
                Case Else
                    AddLog("Thread UDP", "Exception rseau non gre (numro {0} ({1}) : {2}", CStr(DirectCast(pEx, Net.Sockets.SocketException).NativeErrorCode), CStr(DirectCast(pEx, Net.Sockets.SocketException).SocketErrorCode), pEx.ToString, EPrio.Avertissement)
                    MessageBox.Show("Exception non gre dans le thread de rception UDP : " + pEx.ToString + vbCrLf + "n" + CStr(DirectCast(pEx, Net.Sockets.SocketException).NativeErrorCode) + " {" + CStr(DirectCast(pEx, Net.Sockets.SocketException).SocketErrorCode) + "}", "Erreur UDP", MessageBoxButtons.OK, MessageBoxIcon.Exclamation)
                    lRestart = True
            End Select
        Else
            AddLog("Thread UDP", "Exception non gre : " + pEx.ToString, EPrio.Avertissement)
            MessageBox.Show("Exception non gre dans le thread de rception UDP : " + pEx.Message, "Erreur UDP", MessageBoxButtons.OK, MessageBoxIcon.Exclamation)
            lCloseApp = True
        End If

        CloseConnect()
        If lCloseApp Then
            Application.Exit()
            Return False
        End If
        Return lRestart
    End Function


    Private Sub CloseConnect()
        Try
            'Suppression des nom expirs
            RefreshState()
            If mUdp IsNot Nothing Then
                Dim ltData As New List(Of Byte)
                ltData.Add(2)
                ltData.AddRange(GetByteArray(Pseudo))
                ltData.Add(0)
                Send(ltData)
            End If
        Catch ex As Exception
#If DEBUG Then
            Debug.Print("ClsUdp.SubThread.Finally" + vbCrLf + ex.ToString + vbCrLf + "FinMsg")
#End If
            AddLog("Thread UDP.Finally", "Exception non gre : " + ex.ToString, EPrio.Avertissement)

        Finally
            If mUdp IsNot Nothing Then
                mUdp.Close()
                mUdp = Nothing
            End If
        End Try
    End Sub
#End Region
End Class
